"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseCredentialProtocol = void 0;
const EventEmitter_1 = require("../../../agent/EventEmitter");
const didcomm_1 = require("../../../storage/didcomm");
const connections_1 = require("../../connections");
const CredentialEvents_1 = require("../CredentialEvents");
const CredentialState_1 = require("../models/CredentialState");
const repository_1 = require("../repository");
/**
 * Base implementation of the CredentialProtocol that can be used as a foundation for implementing
 * the CredentialProtocol interface.
 */
class BaseCredentialProtocol {
    /**
     * Process a received credential {@link ProblemReportMessage}.
     *
     * @param messageContext The message context containing a credential problem report message
     * @returns credential record associated with the credential problem report message
     */
    async processProblemReport(messageContext) {
        const { message: credentialProblemReportMessage, agentContext, connection } = messageContext;
        const connectionService = agentContext.dependencyManager.resolve(connections_1.ConnectionService);
        agentContext.config.logger.debug(`Processing problem report with message id ${credentialProblemReportMessage.id}`);
        const credentialRecord = await this.getByProperties(agentContext, {
            threadId: credentialProblemReportMessage.threadId,
        });
        // Assert
        await connectionService.assertConnectionOrOutOfBandExchange(messageContext, {
            expectedConnectionId: credentialRecord.connectionId,
        });
        //  This makes sure that the sender of the incoming message is authorized to do so.
        if (!(credentialRecord === null || credentialRecord === void 0 ? void 0 : credentialRecord.connectionId)) {
            await connectionService.matchIncomingMessageToRequestMessageInOutOfBandExchange(messageContext, {
                expectedConnectionId: credentialRecord === null || credentialRecord === void 0 ? void 0 : credentialRecord.connectionId,
            });
            credentialRecord.connectionId = connection === null || connection === void 0 ? void 0 : connection.id;
        }
        // Update record
        credentialRecord.errorMessage = `${credentialProblemReportMessage.description.code}: ${credentialProblemReportMessage.description.en}`;
        await this.updateState(agentContext, credentialRecord, CredentialState_1.CredentialState.Abandoned);
        return credentialRecord;
    }
    /**
     * Update the record to a new state and emit an state changed event. Also updates the record
     * in storage.
     *
     * @param credentialRecord The credential record to update the state for
     * @param newState The state to update to
     *
     */
    async updateState(agentContext, credentialRecord, newState) {
        const credentialRepository = agentContext.dependencyManager.resolve(repository_1.CredentialRepository);
        agentContext.config.logger.debug(`Updating credential record ${credentialRecord.id} to state ${newState} (previous=${credentialRecord.state})`);
        const previousState = credentialRecord.state;
        credentialRecord.state = newState;
        await credentialRepository.update(agentContext, credentialRecord);
        this.emitStateChangedEvent(agentContext, credentialRecord, previousState);
    }
    emitStateChangedEvent(agentContext, credentialRecord, previousState) {
        const eventEmitter = agentContext.dependencyManager.resolve(EventEmitter_1.EventEmitter);
        eventEmitter.emit(agentContext, {
            type: CredentialEvents_1.CredentialEventTypes.CredentialStateChanged,
            payload: {
                credentialRecord: credentialRecord.clone(),
                previousState: previousState,
            },
        });
    }
    /**
     * Retrieve a credential record by id
     *
     * @param credentialRecordId The credential record id
     * @throws {RecordNotFoundError} If no record is found
     * @return The credential record
     *
     */
    getById(agentContext, credentialRecordId) {
        const credentialRepository = agentContext.dependencyManager.resolve(repository_1.CredentialRepository);
        return credentialRepository.getById(agentContext, credentialRecordId);
    }
    /**
     * Retrieve all credential records
     *
     * @returns List containing all credential records
     */
    getAll(agentContext) {
        const credentialRepository = agentContext.dependencyManager.resolve(repository_1.CredentialRepository);
        return credentialRepository.getAll(agentContext);
    }
    async findAllByQuery(agentContext, query, queryOptions) {
        const credentialRepository = agentContext.dependencyManager.resolve(repository_1.CredentialRepository);
        return credentialRepository.findByQuery(agentContext, query, queryOptions);
    }
    /**
     * Find a credential record by id
     *
     * @param credentialRecordId the credential record id
     * @returns The credential record or null if not found
     */
    findById(agentContext, proofRecordId) {
        const credentialRepository = agentContext.dependencyManager.resolve(repository_1.CredentialRepository);
        return credentialRepository.findById(agentContext, proofRecordId);
    }
    async delete(agentContext, credentialRecord, options) {
        var _a, _b;
        const credentialRepository = agentContext.dependencyManager.resolve(repository_1.CredentialRepository);
        const didCommMessageRepository = agentContext.dependencyManager.resolve(didcomm_1.DidCommMessageRepository);
        await credentialRepository.delete(agentContext, credentialRecord);
        const deleteAssociatedCredentials = (_a = options === null || options === void 0 ? void 0 : options.deleteAssociatedCredentials) !== null && _a !== void 0 ? _a : true;
        const deleteAssociatedDidCommMessages = (_b = options === null || options === void 0 ? void 0 : options.deleteAssociatedDidCommMessages) !== null && _b !== void 0 ? _b : true;
        if (deleteAssociatedCredentials) {
            for (const credential of credentialRecord.credentials) {
                const formatService = this.getFormatServiceForRecordType(credential.credentialRecordType);
                await formatService.deleteCredentialById(agentContext, credential.credentialRecordId);
            }
        }
        if (deleteAssociatedDidCommMessages) {
            const didCommMessages = await didCommMessageRepository.findByQuery(agentContext, {
                associatedRecordId: credentialRecord.id,
            });
            for (const didCommMessage of didCommMessages) {
                await didCommMessageRepository.delete(agentContext, didCommMessage);
            }
        }
    }
    /**
     * Retrieve a credential record by connection id and thread id
     *
     * @param properties Properties to query by
     *
     * @throws {RecordNotFoundError} If no record is found
     * @throws {RecordDuplicateError} If multiple records are found
     * @returns The credential record
     */
    getByProperties(agentContext, properties) {
        const { role, connectionId, threadId } = properties;
        const credentialRepository = agentContext.dependencyManager.resolve(repository_1.CredentialRepository);
        return credentialRepository.getSingleByQuery(agentContext, {
            connectionId,
            threadId,
            role,
        });
    }
    /**
     * Find a credential record by connection id and thread id, returns null if not found
     *
     * @param threadId The thread id
     * @param role The role of the record, i.e. holder or issuer
     * @param connectionId The connection id
     *
     * @returns The credential record
     */
    findByProperties(agentContext, properties) {
        const { role, connectionId, threadId } = properties;
        const credentialRepository = agentContext.dependencyManager.resolve(repository_1.CredentialRepository);
        return credentialRepository.findSingleByQuery(agentContext, {
            connectionId,
            threadId,
            role,
        });
    }
    async update(agentContext, credentialRecord) {
        const credentialRepository = agentContext.dependencyManager.resolve(repository_1.CredentialRepository);
        return await credentialRepository.update(agentContext, credentialRecord);
    }
}
exports.BaseCredentialProtocol = BaseCredentialProtocol;
//# sourceMappingURL=BaseCredentialProtocol.js.map